@@ -1,8 +1,10 @@
 import hashlib
 import json
+import os
 import random
 import string
 from collections import namedtuple
+from typing import List
 
 import kubernetes
 from dagster import Array, Field, Noneable, StringSource
@@ -17,9 +19,9 @@ from dagster.utils import frozentags, merge_dicts
 # To retry step job, users should raise RetryRequested() so that the dagster system is aware of the
 # retry. As an example, see retry_pipeline in dagster_test.test_project.test_pipelines.repo
 # To override this config, user can specify UserDefinedDagsterK8sConfig.
-K8S_JOB_BACKOFF_LIMIT = 0
+DEFAULT_K8S_JOB_BACKOFF_LIMIT = 0
 
-K8S_JOB_TTL_SECONDS_AFTER_FINISHED = 24 * 60 * 60  # 1 day
+DEFAULT_K8S_JOB_TTL_SECONDS_AFTER_FINISHED = 24 * 60 * 60  # 1 day
 
 DAGSTER_HOME_DEFAULT = "/opt/dagster/dagster_home"
 
@@ -52,6 +54,11 @@ USER_DEFINED_K8S_CONFIG_SCHEMA = Shape(
     }
 )
 
+DEFAULT_JOB_SPEC_CONFIG = {
+    "ttl_seconds_after_finished": DEFAULT_K8S_JOB_TTL_SECONDS_AFTER_FINISHED,
+    "backoff_limit": DEFAULT_K8S_JOB_BACKOFF_LIMIT,
+}
+
 
 class UserDefinedDagsterK8sConfig(
     namedtuple(
@@ -175,7 +182,8 @@ class DagsterK8sJobConfig(
     namedtuple(
         "_K8sJobTaskConfig",
         "job_image dagster_home image_pull_policy image_pull_secrets service_account_name "
-        "instance_config_map postgres_password_secret env_config_maps env_secrets volume_mounts",
+        "instance_config_map postgres_password_secret env_config_maps env_secrets env_vars "
+        "volume_mounts",
     )
 ):
     """Configuration parameters for launching Dagster Jobs on Kubernetes.
@@ -208,6 +216,8 @@ class DagsterK8sJobConfig(
         env_secrets (Optional[List[str]]): A list of custom Secret names from which to
             draw environment variables (using ``envFrom``) for the Job. Default: ``[]``. See:
             https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables
+        env_vars (Optional[List[str]]): A list of environment variables to inject into the Job.
+            Default: ``[]``. See: https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables
         job_image (Optional[str]): The docker image to use. The Job container will be launched with this
             image. Should not be specified if using userDeployments.
 
@@ -224,6 +234,7 @@ class DagsterK8sJobConfig(
         postgres_password_secret=None,
         env_config_maps=None,
         env_secrets=None,
+        env_vars=None,
         volume_mounts=None,
     ):
         return super(DagsterK8sJobConfig, cls).__new__(
@@ -243,6 +254,7 @@ class DagsterK8sJobConfig(
             ),
             env_config_maps=check.opt_list_param(env_config_maps, "env_config_maps", of_type=str),
             env_secrets=check.opt_list_param(env_secrets, "env_secrets", of_type=str),
+            env_vars=check.opt_list_param(env_vars, "env_secrets", of_type=str),
             volume_mounts=check.opt_list_param(volume_mounts, "volume_mounts"),
         )
 
@@ -283,7 +295,7 @@ class DagsterK8sJobConfig(
         }
 
     @classmethod
-    def config_type_pipeline_run(cls):
+    def config_type_pipeline_run(cls, default_image_pull_policy=None):
         """Configuration intended to be set at pipeline execution time."""
         return {
             "job_image": Field(
@@ -295,11 +307,11 @@ class DagsterK8sJobConfig(
                 '(Ex: "mycompany.com/dagster-k8s-image:latest").',
             ),
             "image_pull_policy": Field(
-                StringSource,
+                Noneable(StringSource),
                 is_required=False,
-                default_value="IfNotPresent",
                 description="Image pull policy to set on the launched task Job Pods. Defaults to "
                 '"IfNotPresent".',
+                default_value=default_image_pull_policy,
             ),
             "image_pull_secrets": Field(
                 Noneable(Array(Shape({"name": StringSource}))),
@@ -327,8 +339,22 @@ class DagsterK8sJobConfig(
                 "variables (using ``envFrom``) for the Job. Default: ``[]``. See:"
                 "https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables",
             ),
+            "env_vars": Field(
+                Noneable(Array(str)),
+                is_required=False,
+                description="A list of environment variables to inject into the Job. "
+                "Default: ``[]``. See: "
+                "https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables",
+            ),
         }
 
+    @property
+    def env(self) -> List[kubernetes.client.V1EnvVar]:
+        return [
+            kubernetes.client.V1EnvVar(name=key, value=os.getenv(key))
+            for key in (self.env_vars or [])
+        ]
+
     @property
     def env_from_sources(self):
         """This constructs a list of env_from sources. Along with a default base environment
@@ -444,12 +470,18 @@ def construct_dagster_k8s_job(
         for key, value in env_vars.items():
             additional_k8s_env_vars.append(kubernetes.client.V1EnvVar(name=key, value=value))
 
+    user_defined_k8s_env_vars = user_defined_k8s_config.container_config.pop("env", [])
+    for env_var in user_defined_k8s_env_vars:
+        additional_k8s_env_vars.append(
+            kubernetes.client.V1EnvVar(name=env_var["name"], value=env_var["value"])
+        )
+
     job_container = kubernetes.client.V1Container(
         name=job_name,
         image=job_config.job_image,
         args=args,
         image_pull_policy=job_config.image_pull_policy,
-        env=env + additional_k8s_env_vars,
+        env=env + job_config.env + additional_k8s_env_vars,
         env_from=job_config.env_from_sources,
         volume_mounts=[
             kubernetes.client.V1VolumeMount(
@@ -516,6 +548,11 @@ def construct_dagster_k8s_job(
         ),
     )
 
+    job_spec_config = merge_dicts(
+        DEFAULT_JOB_SPEC_CONFIG,
+        user_defined_k8s_config.job_spec_config,
+    )
+
     job = kubernetes.client.V1Job(
         api_version="batch/v1",
         kind="Job",
@@ -524,9 +561,7 @@ def construct_dagster_k8s_job(
         ),
         spec=kubernetes.client.V1JobSpec(
             template=template,
-            backoff_limit=K8S_JOB_BACKOFF_LIMIT,
-            ttl_seconds_after_finished=K8S_JOB_TTL_SECONDS_AFTER_FINISHED,
-            **user_defined_k8s_config.job_spec_config,
+            **job_spec_config,
         ),
         **user_defined_k8s_config.job_config,
     )
